package cn.edu.pku.booking.account;

import javax.ejb.Stateful;
import javax.enterprise.inject.Model;
import javax.enterprise.inject.Produces;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.jboss.seam.international.status.Messages;
import org.jboss.seam.international.status.builder.BundleKey;

import cn.edu.pku.booking.i18n.DefaultBundleKey;
import cn.edu.pku.booking.model.User;

/**
 * 注册新用户的controller
 * @author dachao
 */
@Stateful
@Model
public class Registrar {
    @PersistenceContext
    private EntityManager em;

    @Inject
    private Messages messages;

    @Inject
    private FacesContext facesContext;

    private UIInput usernameInput;

    private final User newUser = new User();

    @NotNull
    @Size(min = 5, max = 15)
    private String confirmPassword;

    private boolean registered;

    private boolean registrationInvalid;

    public void register() {
        if (verifyUsernameIsAvailable()) {
            registered = true;
            em.persist(newUser);

            messages.info(new DefaultBundleKey("registration_registered"))
                    .defaults("You have been successfully registered as the user {0}! You can now login.")
                    .params(newUser.getUsername());
        } else {
            registrationInvalid = true;
        }
    }

    public boolean isRegistrationInvalid() {
        return registrationInvalid;
    }

    public void notifyIfRegistrationIsInvalid() {
        if (facesContext.isValidationFailed() || registrationInvalid) {
            messages.warn(new DefaultBundleKey("registration_invalid")).defaults(
                    "Invalid registration. Please correct the errors and try again.");
        }
    }

    @Produces
    @Named
    public User getNewUser() {
        return newUser;
    }

    public boolean isRegistered() {
        return registered;
    }

    public String getConfirmPassword() {
        return confirmPassword;
    }

    public void setConfirmPassword(final String password) {
        confirmPassword = password;
    }

    public UIInput getUsernameInput() {
        return usernameInput;
    }

    public void setUsernameInput(final UIInput usernameInput) {
        this.usernameInput = usernameInput;
    }

    private boolean verifyUsernameIsAvailable() {
        User existing = em.find(User.class, newUser.getUsername());
        if (existing != null) {
            messages.warn(new BundleKey("messages", "account_usernameTaken"))
                    .defaults("The username '{0}' is already taken. Please choose another username.")
                    .targets(usernameInput.getClientId()).params(newUser.getUsername());
            return false;
        }

        return true;
    }

}
