package cn.edu.pku.booking.book;

import static javax.persistence.PersistenceContextType.EXTENDED;

import java.util.HashMap;
import java.util.Locale;

import javax.ejb.Stateful;
import javax.enterprise.context.Conversation;
import javax.enterprise.context.ConversationScoped;
import javax.enterprise.context.RequestScoped;
import javax.enterprise.event.Event;
import javax.enterprise.event.Observes;
import javax.enterprise.event.TransactionPhase;
import javax.enterprise.inject.Produces;
import javax.inject.Inject;
import javax.inject.Named;
import javax.interceptor.Interceptors;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.jboss.seam.faces.context.conversation.Begin;
import org.jboss.seam.faces.context.conversation.ConversationBoundaryInterceptor;
import org.jboss.seam.faces.context.conversation.End;
import org.jboss.seam.international.status.Messages;
import org.jboss.solder.logging.TypedCategory;

import cn.edu.pku.booking.account.Authenticated;
import cn.edu.pku.booking.i18n.DefaultBundleKey;
import cn.edu.pku.booking.log.BookingLog;
import cn.edu.pku.booking.model.Booking;
import cn.edu.pku.booking.model.Hotel;
import cn.edu.pku.booking.model.User;

import com.ocpsoft.pretty.time.PrettyTime;


@Stateful
@ConversationScoped
@Named
@Interceptors(ConversationBoundaryInterceptor.class) 
public class BookingAgent {
    @Inject
    @TypedCategory(BookingAgent.class)
    private BookingLog log;

    @PersistenceContext(type = EXTENDED)
    private EntityManager em;

    @Inject
    private Messages messages;

    @Inject
    @Authenticated
    private User user;

    @Inject
    private Locale locale;

    @Inject
    @Confirmed
    private Event<Booking> bookingConfirmedEventSrc;

    private Hotel hotelSelection;

    private Booking booking;

    private boolean bookingValid;

    @Inject
    private Conversation conversation;

    @Begin
    public void selectHotel(final Long id) {
        conversation.setTimeout(600000); //10 * 60 * 1000 (10 minutes)
        
        hotelSelection = em.find(Hotel.class, id);
        if (hotelSelection != null) {
            log.hotelSelected(user != null ? user.getName() : "Anonymous", hotelSelection.getName(), hotelSelection.getCity());
        }
    }

    public void bookHotel() {
        booking = new Booking(hotelSelection, user, 7, 2);
        hotelSelection = null;

        booking.setCreditCardNumber("1111222233334444");
        log.bookingInitiated(user.getName(), booking.getHotel().getName());

        messages.info(new DefaultBundleKey("booking_initiated")).defaults("You've initiated a booking at the {0}.")
                .params(booking.getHotel().getName());
    }

    public void validate() {
        log.hotelEntityInPersistenceContext(em.contains(booking.getHotel()));
        bookingValid = true;
    }

    @End
    public void confirm() {
        em.persist(booking);
        bookingConfirmedEventSrc.fire(booking);
    }

    @End
    public void cancel() {
        booking = null;
        hotelSelection = null;
    }

    public void onBookingComplete(@Observes(during = TransactionPhase.AFTER_SUCCESS) @Confirmed final Booking booking) {
        log.bookingConfirmed(booking.getHotel().getName(), booking.getUser().getName());
        messages.info(new DefaultBundleKey("booking_confirmed")).defaults("You're booked to stay at the {0} {1}.")
                .params(booking.getHotel().getName(), new PrettyTime(locale).format(booking.getCheckinDate()));
    }

    @Produces
    @ConversationScoped
    @Named
    public Booking getBooking() {
        return booking;
    }

    @Produces
    @RequestScoped
    @Named("hotel")
    public Hotel getSelectedHotel() {
        return booking != null ? booking.getHotel() : hotelSelection;
    }

    public boolean isBookingValid() {
        return bookingValid;
    }
}
